defmodule DecentralisedBookIndex.SyncTasks.SyncBooksTask do
  alias DecentralisedBookIndex.Sync.ApiClients.FetchJsons
  alias DecentralisedBookIndex.Sync.DataTransformers.BookTransformer
  alias DecentralisedBookIndex.Sync.BookSync

  alias DecentralisedBookIndex.Metadata.DBIServer

  require Logger

  def sync(%DBIServer{} = server) do
    url = "#{server.url}/api/v1/json/books"
    FetchJsons.get(url, sync_author_closure(server))

    server
  end

  def sync_author_chunk(json_chunk, server_id) do
    for json <- json_chunk do
      with {:ok, attrs} <- BookTransformer.from_json(json),
           :ok <- BookSync.create_update(attrs, server_id) do
        :ok
      else
        {:error, reason} ->
          Logger.error("Pipeline error: #{inspect(reason)}")
      end
    end

    []
  end

  def sync_author_closure(server) do
    fn json_chunk ->
      sync_author_chunk(json_chunk, server.id)
    end
  end
end
